@@ -1,54 +1,68 @@
 import os
 import sys
+import builtins
 import difflib
-import __builtin__
-import re
-import pydoc
-import contextlib
 import inspect
+import pydoc
 import keyword
-import pkgutil
+import re
+import string
+import test.support
+import time
 import unittest
 import xml.etree
-import types
-import test.test_support
+import textwrap
+from io import StringIO
 from collections import namedtuple
 from test.script_helper import assert_python_ok
-from test.test_support import (
-    TESTFN, rmtree, reap_children, captured_stdout, captured_stderr)
-
+from test.support import (
+    TESTFN, rmtree,
+    reap_children, reap_threads, captured_output, captured_stdout, unlink
+)
 from test import pydoc_mod
 
-if test.test_support.HAVE_DOCSTRINGS:
+try:
+    import threading
+except ImportError:
+    threading = None
+
+# Just in case sys.modules["test"] has the optional attribute __loader__.
+if hasattr(pydoc_mod, "__loader__"):
+    del pydoc_mod.__loader__
+
+if test.support.HAVE_DOCSTRINGS:
     expected_data_docstrings = (
         'dictionary for instance variables (if defined)',
         'list of weak references to the object (if defined)',
-        )
+        ) * 2
 else:
-    expected_data_docstrings = ('', '')
+    expected_data_docstrings = ('', '', '', '')
 
-expected_text_pattern = \
-"""
+expected_text_pattern = """
 NAME
     test.pydoc_mod - This is a test module for test_pydoc
-
-FILE
-    %s
 %s
 CLASSES
-    __builtin__.object
+    builtins.object
+        A
         B
-    A
 \x20\x20\x20\x20
-    class A
+    class A(builtins.object)
      |  Hello and goodbye
      |\x20\x20
      |  Methods defined here:
      |\x20\x20
      |  __init__()
      |      Wow, I have no function!
+     |\x20\x20
+     |  ----------------------------------------------------------------------
+     |  Data descriptors defined here:
+     |\x20\x20
+     |  __dict__%s
+     |\x20\x20
+     |  __weakref__%s
 \x20\x20\x20\x20
-    class B(__builtin__.object)
+    class B(builtins.object)
      |  Data descriptors defined here:
      |\x20\x20
      |  __dict__%s
@@ -70,9 +84,7 @@ FUNCTIONS
     nodoc_func()
 
 DATA
-    __author__ = 'Benjamin Peterson'
-    __credits__ = 'Nobody'
-    __version__ = '1.2.3.4'
+    __xyz__ = 'X, Y and Z'
 
 VERSION
     1.2.3.4
@@ -82,13 +94,15 @@ AUTHOR
 
 CREDITS
     Nobody
+
+FILE
+    %s
 """.strip()
 
 expected_text_data_docstrings = tuple('\n     |      ' + s if s else ''
                                       for s in expected_data_docstrings)
 
-expected_html_pattern = \
-"""
+expected_html_pattern = """
 <table width="100%%" cellspacing=0 cellpadding=2 border=0 summary="heading">
 <tr bgcolor="#7799ee">
 <td valign=bottom>&nbsp;<br>
@@ -104,19 +118,19 @@ expected_html_pattern = \
 \x20\x20\x20\x20
 <tr><td bgcolor="#ee77aa"><tt>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</tt></td><td>&nbsp;</td>
 <td width="100%%"><dl>
-<dt><font face="helvetica, arial"><a href="__builtin__.html#object">__builtin__.object</a>
+<dt><font face="helvetica, arial"><a href="builtins.html#object">builtins.object</a>
 </font></dt><dd>
 <dl>
-<dt><font face="helvetica, arial"><a href="test.pydoc_mod.html#B">B</a>
+<dt><font face="helvetica, arial"><a href="test.pydoc_mod.html#A">A</a>
+</font></dt><dt><font face="helvetica, arial"><a href="test.pydoc_mod.html#B">B</a>
 </font></dt></dl>
 </dd>
-<dt><font face="helvetica, arial"><a href="test.pydoc_mod.html#A">A</a>
-</font></dt></dl>
+</dl>
  <p>
 <table width="100%%" cellspacing=0 cellpadding=2 border=0 summary="section">
 <tr bgcolor="#ffc8d8">
 <td colspan=3 valign=bottom>&nbsp;<br>
-<font color="#000000" face="helvetica, arial"><a name="A">class <strong>A</strong></a></font></td></tr>
+<font color="#000000" face="helvetica, arial"><a name="A">class <strong>A</strong></a>(<a href="builtins.html#object">builtins.object</a>)</font></td></tr>
 \x20\x20\x20\x20
 <tr bgcolor="#ffc8d8"><td rowspan=2><tt>&nbsp;&nbsp;&nbsp;</tt></td>
 <td colspan=2><tt>Hello&nbsp;and&nbsp;goodbye<br>&nbsp;</tt></td></tr>
@@ -124,11 +138,19 @@ expected_html_pattern = \
 <td width="100%%">Methods defined here:<br>
 <dl><dt><a name="A-__init__"><strong>__init__</strong></a>()</dt><dd><tt>Wow,&nbsp;I&nbsp;have&nbsp;no&nbsp;function!</tt></dd></dl>
 
+<hr>
+Data descriptors defined here:<br>
+<dl><dt><strong>__dict__</strong></dt>
+<dd><tt>%s</tt></dd>
+</dl>
+<dl><dt><strong>__weakref__</strong></dt>
+<dd><tt>%s</tt></dd>
+</dl>
 </td></tr></table> <p>
 <table width="100%%" cellspacing=0 cellpadding=2 border=0 summary="section">
 <tr bgcolor="#ffc8d8">
 <td colspan=3 valign=bottom>&nbsp;<br>
-<font color="#000000" face="helvetica, arial"><a name="B">class <strong>B</strong></a>(<a href="__builtin__.html#object">__builtin__.object</a>)</font></td></tr>
+<font color="#000000" face="helvetica, arial"><a name="B">class <strong>B</strong></a>(<a href="builtins.html#object">builtins.object</a>)</font></td></tr>
 \x20\x20\x20\x20
 <tr><td bgcolor="#ffc8d8"><tt>&nbsp;&nbsp;&nbsp;</tt></td><td>&nbsp;</td>
 <td width="100%%">Data descriptors defined here:<br>
@@ -161,9 +183,7 @@ war</tt></dd></dl>
 <font color="#ffffff" face="helvetica, arial"><big><strong>Data</strong></big></font></td></tr>
 \x20\x20\x20\x20
 <tr><td bgcolor="#55aa55"><tt>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</tt></td><td>&nbsp;</td>
-<td width="100%%"><strong>__author__</strong> = 'Benjamin Peterson'<br>
-<strong>__credits__</strong> = 'Nobody'<br>
-<strong>__version__</strong> = '1.2.3.4'</td></tr></table><p>
+<td width="100%%"><strong>__xyz__</strong> = 'X, Y and Z'</td></tr></table><p>
 <table width="100%%" cellspacing=0 cellpadding=2 border=0 summary="section">
 <tr bgcolor="#7799ee">
 <td colspan=3 valign=bottom>&nbsp;<br>
@@ -178,7 +198,7 @@ war</tt></dd></dl>
 \x20\x20\x20\x20
 <tr><td bgcolor="#7799ee"><tt>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</tt></td><td>&nbsp;</td>
 <td width="100%%">Nobody</td></tr></table>
-""".strip()
+""".strip() # ' <- emacs turd
 
 expected_html_data_docstrings = tuple(s.replace(' ', '&nbsp;')
                                       for s in expected_data_docstrings)
@@ -187,7 +207,7 @@ expected_html_data_docstrings = tuple(s.replace(' ', '&nbsp;')
 missing_pattern = "no Python documentation found for '%s'"
 
 # output pattern for module with bad imports
-badimport_pattern = "problem in %s - <type 'exceptions.ImportError'>: No module named %s"
+badimport_pattern = "problem in %s - ImportError: No module named %s"
 
 def run_pydoc(module_name, *args, **env):
     """
@@ -217,41 +237,26 @@ def get_pydoc_text(module):
 
     output = doc.docmodule(module)
 
-    # cleanup the extra text formatting that pydoc preforms
+    # clean up the extra text formatting that pydoc performs
     patt = re.compile('\b.')
     output = patt.sub('', output)
     return output.strip(), loc
 
 def print_diffs(text1, text2):
     "Prints unified diffs for two texts"
+    # XXX now obsolete, use unittest built-in support
     lines1 = text1.splitlines(True)
     lines2 = text2.splitlines(True)
     diffs = difflib.unified_diff(lines1, lines2, n=0, fromfile='expected',
                                  tofile='got')
-    print '\n' + ''.join(diffs)
+    print('\n' + ''.join(diffs))
 
-
-class PydocBaseTest(unittest.TestCase):
-
-    def _restricted_walk_packages(self, walk_packages, path=None):
-        """
-        A version of pkgutil.walk_packages() that will restrict itself to
-        a given path.
-        """
-        default_path = path or [os.path.dirname(__file__)]
-        def wrapper(path=None, prefix='', onerror=None):
-            return walk_packages(path or default_path, prefix, onerror)
-        return wrapper
-
-    @contextlib.contextmanager
-    def restrict_walk_packages(self, path=None):
-        walk_packages = pkgutil.walk_packages
-        pkgutil.walk_packages = self._restricted_walk_packages(walk_packages,
-                                                               path)
-        try:
-            yield
-        finally:
-            pkgutil.walk_packages = walk_packages
+def get_html_title(text):
+    # Bit of hack, but good enough for test purposes
+    header, _, _ = text.partition("</head>")
+    _, _, title = header.partition("<title>")
+    title, _, _ = title.partition("</title>")
+    return title
 
 
 class PydocDocTest(unittest.TestCase):
@@ -278,8 +283,9 @@ class PydocDocTest(unittest.TestCase):
     def test_text_doc(self):
         result, doc_loc = get_pydoc_text(pydoc_mod)
         expected_text = expected_text_pattern % (
-                        (inspect.getabsfile(pydoc_mod), doc_loc) +
-                        expected_text_data_docstrings)
+                        (doc_loc,) +
+                        expected_text_data_docstrings +
+                        (inspect.getabsfile(pydoc_mod),))
         if result != expected_text:
             print_diffs(expected_text, result)
             self.fail("outputs are not equal, see diff above")
@@ -302,18 +308,16 @@ class PydocDocTest(unittest.TestCase):
 
     def test_not_here(self):
         missing_module = "test.i_am_not_here"
-        result = run_pydoc(missing_module)
+        result = str(run_pydoc(missing_module), 'ascii')
         expected = missing_pattern % missing_module
         self.assertEqual(expected, result,
             "documentation for missing module found")
 
     def test_input_strip(self):
         missing_module = " test.i_am_not_here "
-        result = run_pydoc(missing_module)
+        result = str(run_pydoc(missing_module), 'ascii')
         expected = missing_pattern % missing_module.strip()
-        self.assertEqual(expected, result,
-            "white space was not stripped from module name "
-            "or other error output mismatch")
+        self.assertEqual(expected, result)
 
     def test_stripid(self):
         # test with strings, other implementations might have different repr()
@@ -328,8 +332,89 @@ class PydocDocTest(unittest.TestCase):
         self.assertEqual(stripid("<type 'exceptions.Exception'>"),
                          "<type 'exceptions.Exception'>")
 
+    @unittest.skipIf(sys.flags.optimize >= 2,
+                     'Docstrings are omitted with -O2 and above')
+    def test_help_output_redirect(self):
+        # issue 940286, if output is set in Helper, then all output from
+        # Helper.help should be redirected
+        old_pattern = expected_text_pattern
+        getpager_old = pydoc.getpager
+        getpager_new = lambda: (lambda x: x)
+        self.maxDiff = None
 
-class PydocImportTest(PydocBaseTest):
+        buf = StringIO()
+        helper = pydoc.Helper(output=buf)
+        unused, doc_loc = get_pydoc_text(pydoc_mod)
+        module = "test.pydoc_mod"
+        help_header = """
+        Help on module test.pydoc_mod in test:
+
+        """.lstrip()
+        help_header = textwrap.dedent(help_header)
+        expected_help_pattern = help_header + expected_text_pattern
+
+        pydoc.getpager = getpager_new
+        try:
+            with captured_output('stdout') as output, \
+                 captured_output('stderr') as err:
+                helper.help(module)
+                result = buf.getvalue().strip()
+                expected_text = expected_help_pattern % (
+                                (doc_loc,) +
+                                expected_text_data_docstrings +
+                                (inspect.getabsfile(pydoc_mod),))
+                self.assertEqual('', output.getvalue())
+                self.assertEqual('', err.getvalue())
+                self.assertEqual(expected_text, result)
+        finally:
+            pydoc.getpager = getpager_old
+
+    def test_namedtuple_public_underscore(self):
+        NT = namedtuple('NT', ['abc', 'def'], rename=True)
+        with captured_stdout() as help_io:
+            help(NT)
+        helptext = help_io.getvalue()
+        self.assertIn('_1', helptext)
+        self.assertIn('_replace', helptext)
+        self.assertIn('_asdict', helptext)
+
+    def test_synopsis(self):
+        self.addCleanup(unlink, TESTFN)
+        for encoding in ('ISO-8859-1', 'UTF-8'):
+            with open(TESTFN, 'w', encoding=encoding) as script:
+                if encoding != 'UTF-8':
+                    print('#coding: {}'.format(encoding), file=script)
+                print('"""line 1: h\xe9', file=script)
+                print('line 2: hi"""', file=script)
+            synopsis = pydoc.synopsis(TESTFN, {})
+            self.assertEqual(synopsis, 'line 1: h\xe9')
+
+    def test_allmethods(self):
+        # issue 17476: allmethods was no longer returning unbound methods.
+        # This test is a bit fragile in the face of changes to object and type,
+        # but I can't think of a better way to do it without duplicating the
+        # logic of the function under test.
+
+        class TestClass(object):
+            def method_returning_true(self):
+                return True
+
+        # What we expect to get back: everything on object...
+        expected = dict(vars(object))
+        # ...plus our unbound method...
+        expected['method_returning_true'] = TestClass.method_returning_true
+        # ...but not the non-methods on object.
+        del expected['__doc__']
+        del expected['__class__']
+        # inspect resolves descriptors on type into methods, but vars doesn't,
+        # so we need to update __subclasshook__.
+        expected['__subclasshook__'] = TestClass.__subclasshook__
+
+        methods = pydoc.allmethods(TestClass)
+        self.assertDictEqual(methods, expected)
+
+
+class PydocImportTest(unittest.TestCase):
 
     def setUp(self):
         self.test_dir = os.mkdir(TESTFN)
@@ -353,7 +438,7 @@ class PydocImportTest(PydocBaseTest):
         for importstring, expectedinmsg in testpairs:
             with open(sourcefn, 'w') as f:
                 f.write("import {}\n".format(importstring))
-            result = run_pydoc(modname, PYTHONPATH=TESTFN)
+            result = run_pydoc(modname, PYTHONPATH=TESTFN).decode("ascii")
             expected = badimport_pattern % (modname, expectedinmsg)
             self.assertEqual(expected, result)
 
@@ -364,19 +449,8 @@ class PydocImportTest(PydocBaseTest):
         badsyntax = os.path.join(pkgdir, "__init__") + os.extsep + "py"
         with open(badsyntax, 'w') as f:
             f.write("invalid python syntax = $1\n")
-        with self.restrict_walk_packages(path=[TESTFN]):
-            with captured_stdout() as out:
-                with captured_stderr() as err:
-                    pydoc.apropos('xyzzy')
-            # No result, no error
-            self.assertEqual(out.getvalue(), '')
-            self.assertEqual(err.getvalue(), '')
-            # The package name is still matched
-            with captured_stdout() as out:
-                with captured_stderr() as err:
-                    pydoc.apropos('syntaxerr')
-            self.assertEqual(out.getvalue().strip(), 'syntaxerr')
-            self.assertEqual(err.getvalue(), '')
+        result = run_pydoc('zqwykjv', '-k', PYTHONPATH=TESTFN)
+        self.assertEqual(b'', result)
 
     def test_apropos_with_unreadable_dir(self):
         # Issue 7367 - pydoc -k failed when unreadable dir on path
@@ -385,13 +459,8 @@ class PydocImportTest(PydocBaseTest):
         self.addCleanup(os.rmdir, self.unreadable_dir)
         # Note, on Windows the directory appears to be still
         #   readable so this is not really testing the issue there
-        with self.restrict_walk_packages(path=[TESTFN]):
-            with captured_stdout() as out:
-                with captured_stderr() as err:
-                    pydoc.apropos('SOMEKEY')
-        # No result, no error
-        self.assertEqual(out.getvalue(), '')
-        self.assertEqual(err.getvalue(), '')
+        result = run_pydoc('zqwykjv', '-k', PYTHONPATH=TESTFN)
+        self.assertEqual(b'', result)
 
 
 class TestDescriptions(unittest.TestCase):
@@ -402,16 +471,8 @@ class TestDescriptions(unittest.TestCase):
         doc = pydoc.render_doc(pydocfodder)
         self.assertIn("pydocfodder", doc)
 
-    def test_classic_class(self):
-        class C: "Classic class"
-        c = C()
-        self.assertEqual(pydoc.describe(C), 'class C')
-        self.assertEqual(pydoc.describe(c), 'instance of C')
-        expected = 'instance of C in module %s' % __name__
-        self.assertIn(expected, pydoc.render_doc(c))
-
     def test_class(self):
-        class C(object): "New-style class"
+        class C: "New-style class"
         c = C()
 
         self.assertEqual(pydoc.describe(C), 'class C')
@@ -419,114 +480,9 @@ class TestDescriptions(unittest.TestCase):
         expected = 'C in module %s object' % __name__
         self.assertIn(expected, pydoc.render_doc(c))
 
-    def test_namedtuple_public_underscore(self):
-        NT = namedtuple('NT', ['abc', 'def'], rename=True)
-        with captured_stdout() as help_io:
-            pydoc.help(NT)
-        helptext = help_io.getvalue()
-        self.assertIn('_1', helptext)
-        self.assertIn('_replace', helptext)
-        self.assertIn('_asdict', helptext)
-
-
-@unittest.skipUnless(test.test_support.have_unicode,
-                     "test requires unicode support")
-class TestUnicode(unittest.TestCase):
-
-    def setUp(self):
-        # Better not to use unicode escapes in literals, lest the
-        # parser choke on it if Python has been built without
-        # unicode support.
-        self.Q  = types.ModuleType(
-            'Q', 'Rational numbers: \xe2\x84\x9a'.decode('utf8'))
-        self.Q.__version__ = '\xe2\x84\x9a'.decode('utf8')
-        self.Q.__date__ = '\xe2\x84\x9a'.decode('utf8')
-        self.Q.__author__ = '\xe2\x84\x9a'.decode('utf8')
-        self.Q.__credits__ = '\xe2\x84\x9a'.decode('utf8')
-
-        self.assertIsInstance(self.Q.__doc__, unicode)
-
-    def test_render_doc(self):
-        # render_doc is robust against unicode in docstrings
-        doc = pydoc.render_doc(self.Q)
-        self.assertIsInstance(doc, str)
-
-    def test_encode(self):
-        # _encode is robust against characters out the specified encoding
-        self.assertEqual(pydoc._encode(self.Q.__doc__, 'ascii'), 'Rational numbers: &#8474;')
-
-    def test_pipepager(self):
-        # pipepager does not choke on unicode
-        doc = pydoc.render_doc(self.Q)
-
-        saved, os.popen = os.popen, open
-        try:
-            with test.test_support.temp_cwd():
-                pydoc.pipepager(doc, 'pipe')
-                self.assertEqual(open('pipe').read(), pydoc._encode(doc))
-        finally:
-            os.popen = saved
-
-    def test_tempfilepager(self):
-        # tempfilepager does not choke on unicode
-        doc = pydoc.render_doc(self.Q)
-
-        output = {}
-        def mock_system(cmd):
-            filename = cmd.strip()[1:-1]
-            self.assertEqual('"' + filename + '"', cmd.strip())
-            output['content'] = open(filename).read()
-        saved, os.system = os.system, mock_system
-        try:
-            pydoc.tempfilepager(doc, '')
-            self.assertEqual(output['content'], pydoc._encode(doc))
-        finally:
-            os.system = saved
-
-    def test_plainpager(self):
-        # plainpager does not choke on unicode
-        doc = pydoc.render_doc(self.Q)
-
-        # Note: captured_stdout is too permissive when it comes to
-        # unicode, and using it here would make the test always
-        # pass.
-        with test.test_support.temp_cwd():
-            with open('output', 'w') as f:
-                saved, sys.stdout = sys.stdout, f
-                try:
-                    pydoc.plainpager(doc)
-                finally:
-                    sys.stdout = saved
-            self.assertIn('Rational numbers:', open('output').read())
-
-    def test_ttypager(self):
-        # ttypager does not choke on unicode
-        doc = pydoc.render_doc(self.Q)
-        # Test ttypager
-        with test.test_support.temp_cwd(), test.test_support.captured_stdin():
-            with open('output', 'w') as f:
-                saved, sys.stdout = sys.stdout, f
-                try:
-                    pydoc.ttypager(doc)
-                finally:
-                    sys.stdout = saved
-            self.assertIn('Rational numbers:', open('output').read())
-
-    def test_htmlpage(self):
-        # html.page does not choke on unicode
-        with test.test_support.temp_cwd():
-            with captured_stdout() as output:
-                pydoc.writedoc(self.Q)
-        self.assertEqual(output.getvalue(), 'wrote Q.html\n')
-
-class TestHelper(unittest.TestCase):
-    def test_keywords(self):
-        self.assertEqual(sorted(pydoc.Helper.keywords),
-                         sorted(keyword.kwlist))
-
     def test_builtin(self):
-        for name in ('str', 'str.translate', '__builtin__.str',
-                     '__builtin__.str.translate'):
+        for name in ('str', 'str.translate', 'builtins.str',
+                     'builtins.str.translate'):
             # test low-level function
             self.assertIsNotNone(pydoc.locate(name))
             # test high-level function
@@ -535,20 +491,94 @@ class TestHelper(unittest.TestCase):
             except ImportError:
                 self.fail('finding the doc of {!r} failed'.format(o))
 
-        for name in ('not__builtin__', 'strrr', 'strr.translate',
-                     'str.trrrranslate', '__builtin__.strrr',
-                     '__builtin__.str.trrranslate'):
+        for name in ('notbuiltins', 'strrr', 'strr.translate',
+                     'str.trrrranslate', 'builtins.strrr',
+                     'builtins.str.trrranslate'):
             self.assertIsNone(pydoc.locate(name))
             self.assertRaises(ImportError, pydoc.render_doc, name)
 
 
+@unittest.skipUnless(threading, 'Threading required for this test.')
+class PydocServerTest(unittest.TestCase):
+    """Tests for pydoc._start_server"""
+
+    def test_server(self):
+
+        # Minimal test that starts the server, then stops it.
+        def my_url_handler(url, content_type):
+            text = 'the URL sent was: (%s, %s)' % (url, content_type)
+            return text
+
+        serverthread = pydoc._start_server(my_url_handler, port=0)
+        self.assertIn('localhost', serverthread.docserver.address)
+
+        starttime = time.time()
+        timeout = 1  #seconds
+
+        while serverthread.serving:
+            time.sleep(.01)
+            if serverthread.serving and time.time() - starttime > timeout:
+                serverthread.stop()
+                break
+
+        self.assertEqual(serverthread.error, None)
+
+
+class PydocUrlHandlerTest(unittest.TestCase):
+    """Tests for pydoc._url_handler"""
+
+    def test_content_type_err(self):
+        f = pydoc._url_handler
+        self.assertRaises(TypeError, f, 'A', '')
+        self.assertRaises(TypeError, f, 'B', 'foobar')
+
+    def test_url_requests(self):
+        # Test for the correct title in the html pages returned.
+        # This tests the different parts of the URL handler without
+        # getting too picky about the exact html.
+        requests = [
+            ("", "Pydoc: Index of Modules"),
+            ("get?key=", "Pydoc: Index of Modules"),
+            ("index", "Pydoc: Index of Modules"),
+            ("topics", "Pydoc: Topics"),
+            ("keywords", "Pydoc: Keywords"),
+            ("pydoc", "Pydoc: module pydoc"),
+            ("get?key=pydoc", "Pydoc: module pydoc"),
+            ("search?key=pydoc", "Pydoc: Search Results"),
+            ("topic?key=def", "Pydoc: KEYWORD def"),
+            ("topic?key=STRINGS", "Pydoc: TOPIC STRINGS"),
+            ("foobar", "Pydoc: Error - foobar"),
+            ("getfile?key=foobar", "Pydoc: Error - getfile?key=foobar"),
+            ]
+
+        for url, title in requests:
+            text = pydoc._url_handler(url, "text/html")
+            result = get_html_title(text)
+            self.assertEqual(result, title)
+
+        path = string.__file__
+        title = "Pydoc: getfile " + path
+        url = "getfile?key=" + path
+        text = pydoc._url_handler(url, "text/html")
+        result = get_html_title(text)
+        self.assertEqual(result, title)
+
+
+class TestHelper(unittest.TestCase):
+    def test_keywords(self):
+        self.assertEqual(sorted(pydoc.Helper.keywords),
+                         sorted(keyword.kwlist))
+
+@reap_threads
 def test_main():
     try:
-        test.test_support.run_unittest(PydocDocTest,
-                                       PydocImportTest,
-                                       TestDescriptions,
-                                       TestUnicode,
-                                       TestHelper)
+        test.support.run_unittest(PydocDocTest,
+                                  PydocImportTest,
+                                  TestDescriptions,
+                                  PydocServerTest,
+                                  PydocUrlHandlerTest,
+                                  TestHelper,
+                                  )
     finally:
         reap_children()
 
