@@ -174,7 +174,7 @@ list_join(PyObject* list)
     switch (PyList_GET_SIZE(list)) {
     case 0:
         Py_DECREF(list);
-        return PyString_FromString("");
+        return PyBytes_FromString("");
     case 1:
         result = PyList_GET_ITEM(list, 0);
         Py_INCREF(result);
@@ -251,7 +251,7 @@ typedef struct {
 
 } ElementObject;
 
-staticforward PyTypeObject Element_Type;
+static PyTypeObject Element_Type;
 
 #define Element_CheckExact(op) (Py_TYPE(op) == &Element_Type)
 
@@ -693,7 +693,7 @@ element_deepcopy(ElementObject* self, PyObject* args)
     }
 
     /* add object to memo dictionary (so deepcopy won't visit it again) */
-    id = PyInt_FromLong((Py_uintptr_t) self);
+    id = PyLong_FromLong((Py_uintptr_t) self);
     if (!id)
         goto error;
 
@@ -722,7 +722,6 @@ checkpath(PyObject* tag)
 #define PATHCHAR(ch) \
     (ch == '/' || ch == '*' || ch == '[' || ch == '@' || ch == '.')
 
-#if defined(Py_USING_UNICODE)
     if (PyUnicode_Check(tag)) {
         Py_UNICODE *p = PyUnicode_AS_UNICODE(tag);
         for (i = 0; i < PyUnicode_GET_SIZE(tag); i++) {
@@ -735,10 +734,9 @@ checkpath(PyObject* tag)
         }
         return 0;
     }
-#endif
-    if (PyString_Check(tag)) {
-        char *p = PyString_AS_STRING(tag);
-        for (i = 0; i < PyString_GET_SIZE(tag); i++) {
+    if (PyBytes_Check(tag)) {
+        char *p = PyBytes_AS_STRING(tag);
+        for (i = 0; i < PyBytes_GET_SIZE(tag); i++) {
             if (p[i] == '{')
                 check = 0;
             else if (p[i] == '}')
@@ -756,7 +754,7 @@ static PyObject*
 element_extend(ElementObject* self, PyObject* args)
 {
     PyObject* seq;
-    Py_ssize_t i;
+    Py_ssize_t i, seqlen = 0;
 
     PyObject* seq_in;
     if (!PyArg_ParseTuple(args, "O:extend", &seq_in))
@@ -771,7 +769,8 @@ element_extend(ElementObject* self, PyObject* args)
         return NULL;
     }
 
-    for (i = 0; i < PySequence_Fast_GET_SIZE(seq); i++) {
+    seqlen = PySequence_Size(seq);
+    for (i = 0; i < seqlen; i++) {
         PyObject* element = PySequence_Fast_GET_ITEM(seq, i);
         if (element_add_subelement(self, element) < 0) {
             Py_DECREF(seq);
@@ -804,16 +803,11 @@ element_find(ElementObject* self, PyObject* args)
         
     for (i = 0; i < self->extra->length; i++) {
         PyObject* item = self->extra->children[i];
-        int rc;
-        if (!Element_CheckExact(item))
-            continue;
-        Py_INCREF(item);
-        rc = PyObject_Compare(((ElementObject*)item)->tag, tag);
-        if (rc == 0)
+        if (Element_CheckExact(item) &&
+            PyObject_RichCompareBool(((ElementObject*)item)->tag, tag, Py_EQ) == 1) {
+            Py_INCREF(item);
             return item;
-        Py_DECREF(item);
-        if (rc < 0 && PyErr_Occurred())
-            return NULL;
+        }
     }
 
     Py_RETURN_NONE;
@@ -842,24 +836,14 @@ element_findtext(ElementObject* self, PyObject* args)
 
     for (i = 0; i < self->extra->length; i++) {
         ElementObject* item = (ElementObject*) self->extra->children[i];
-        int rc;
-        if (!Element_CheckExact(item))
-            continue;
-        Py_INCREF(item);
-        rc = PyObject_Compare(item->tag, tag);
-        if (rc == 0) {
+        if (Element_CheckExact(item) && (PyObject_RichCompareBool(item->tag, tag, Py_EQ) == 1)) {
+
             PyObject* text = element_get_text(item);
-            if (text == Py_None) {
-                Py_DECREF(item);
-                return PyString_FromString("");
-            }
+            if (text == Py_None)
+                return PyUnicode_FromString("");
             Py_XINCREF(text);
-            Py_DECREF(item);
             return text;
         }
-        Py_DECREF(item);
-        if (rc < 0 && PyErr_Occurred())
-            return NULL;
     }
 
     Py_INCREF(default_value);
@@ -891,17 +875,12 @@ element_findall(ElementObject* self, PyObject* args)
 
     for (i = 0; i < self->extra->length; i++) {
         PyObject* item = self->extra->children[i];
-        int rc;
-        if (!Element_CheckExact(item))
-            continue;
-        Py_INCREF(item);
-        rc = PyObject_Compare(((ElementObject*)item)->tag, tag);
-        if (rc == 0)
-            rc = PyList_Append(out, item);
-        Py_DECREF(item);
-        if (rc < 0 && PyErr_Occurred()) {
-            Py_DECREF(out);
-            return NULL;
+        if (Element_CheckExact(item) &&
+            PyObject_RichCompareBool(((ElementObject*)item)->tag, tag, Py_EQ) == 1) {
+            if (PyList_Append(out, item) < 0) {
+                Py_DECREF(out);
+                return NULL;
+            }
         }
     }
 
@@ -1167,10 +1146,8 @@ static PyObject*
 element_remove(ElementObject* self, PyObject* args)
 {
     int i;
-    int rc;
-    PyObject* element;
-    PyObject* found;
 
+    PyObject* element;
     if (!PyArg_ParseTuple(args, "O!:remove", &Element_Type, &element))
         return NULL;
 
@@ -1186,14 +1163,11 @@ element_remove(ElementObject* self, PyObject* args)
     for (i = 0; i < self->extra->length; i++) {
         if (self->extra->children[i] == element)
             break;
-        rc = PyObject_Compare(self->extra->children[i], element);
-        if (rc == 0)
+        if (PyObject_RichCompareBool(self->extra->children[i], element, Py_EQ) == 1)
             break;
-        if (rc < 0 && PyErr_Occurred())
-            return NULL;
     }
 
-    if (i >= self->extra->length) {
+    if (i == self->extra->length) {
         /* element is not in children, so raise exception */
         PyErr_SetString(
             PyExc_ValueError,
@@ -1202,31 +1176,20 @@ element_remove(ElementObject* self, PyObject* args)
         return NULL;
     }
 
-    found = self->extra->children[i];
+    Py_DECREF(self->extra->children[i]);
 
     self->extra->length--;
+
     for (; i < self->extra->length; i++)
         self->extra->children[i] = self->extra->children[i+1];
 
-    Py_DECREF(found);
     Py_RETURN_NONE;
 }
 
 static PyObject*
 element_repr(ElementObject* self)
 {
-    PyObject *repr, *tag;
-
-    tag = PyObject_Repr(self->tag);
-    if (!tag)
-        return NULL;
-
-    repr = PyString_FromFormat("<Element %s at %p>",
-                               PyString_AS_STRING(tag), self);
-
-    Py_DECREF(tag);
-
-    return repr;
+    return PyUnicode_FromFormat("<Element %R at %p>", self->tag, self);
 }
 
 static PyObject*
@@ -1309,7 +1272,7 @@ element_subscr(PyObject* self_, PyObject* item)
         if (!self->extra)
             return PyList_New(0);
 
-        if (PySlice_GetIndicesEx((PySliceObject *)item,
+        if (PySlice_GetIndicesEx(item,
                 self->extra->length,
                 &start, &stop, &step, &slicelen) < 0) {
             return NULL;
@@ -1368,7 +1331,7 @@ element_ass_subscr(PyObject* self_, PyObject* item, PyObject* value)
         if (!self->extra)
             element_new_extra(self, NULL);
 
-        if (PySlice_GetIndicesEx((PySliceObject *)item,
+        if (PySlice_GetIndicesEx(item,
                 self->extra->length,
                 &start, &stop, &step, &slicelen) < 0) {
             return -1;
@@ -1512,10 +1475,17 @@ static PyMethodDef element_methods[] = {
     {NULL, NULL}
 };
 
-static PyObject*  
-element_getattr(ElementObject* self, char* name)
+static PyObject*
+element_getattro(ElementObject* self, PyObject* nameobj)
 {
     PyObject* res;
+    char *name = "";
+
+    if (PyUnicode_Check(nameobj))
+        name = _PyUnicode_AsString(nameobj);
+    
+    if (name == NULL)
+        return NULL;
 
     /* handle common attributes first */
     if (strcmp(name, "tag") == 0) {
@@ -1529,22 +1499,19 @@ element_getattr(ElementObject* self, char* name)
     }
 
     /* methods */
-    res = Py_FindMethod(element_methods, (PyObject*) self, name);
+    res = PyObject_GenericGetAttr((PyObject*) self, nameobj);
     if (res)
         return res;
 
-    PyErr_Clear();
-
     /* less common attributes */
     if (strcmp(name, "tail") == 0) {
+        PyErr_Clear();
         res = element_get_tail(self);
     } else if (strcmp(name, "attrib") == 0) {
+        PyErr_Clear();
         if (!self->extra)
             element_new_extra(self, NULL);
         res = element_get_attrib(self);
-    } else {
-        PyErr_SetString(PyExc_AttributeError, name);
-        return NULL;
     }
 
     if (!res)
@@ -1607,19 +1574,35 @@ static PyMappingMethods element_as_mapping = {
     (objobjargproc) element_ass_subscr,
 };
 
-statichere PyTypeObject Element_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0, "Element", sizeof(ElementObject), 0,
+static PyTypeObject Element_Type = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "Element", sizeof(ElementObject), 0,
     /* methods */
     (destructor)element_dealloc, /* tp_dealloc */
     0, /* tp_print */
-    (getattrfunc)element_getattr, /* tp_getattr */
+    0, /* tp_getattr */
     (setattrfunc)element_setattr, /* tp_setattr */
-    0, /* tp_compare */
+    0, /* tp_reserved */
     (reprfunc)element_repr, /* tp_repr */
     0, /* tp_as_number */
     &element_as_sequence, /* tp_as_sequence */
     &element_as_mapping, /* tp_as_mapping */
+    0, /* tp_hash */
+    0, /* tp_call */
+    0, /* tp_str */
+    (getattrofunc)element_getattro, /* tp_getattro */
+    0, /* tp_setattro */
+    0, /* tp_as_buffer */
+    Py_TPFLAGS_DEFAULT, /* tp_flags */
+    0, /* tp_doc */
+    0, /* tp_traverse */
+    0, /* tp_clear */
+    0, /* tp_richcompare */
+    0, /* tp_weaklistoffset */
+    0, /* tp_iter */
+    0, /* tp_iternext */
+    element_methods, /* tp_methods */
+    0, /* tp_members */
 };
 
 /* ==================================================================== */
@@ -1647,7 +1630,7 @@ typedef struct {
 
 } TreeBuilderObject;
 
-staticforward PyTypeObject TreeBuilder_Type;
+static PyTypeObject TreeBuilder_Type;
 
 #define TreeBuilder_CheckExact(op) (Py_TYPE(op) == &TreeBuilder_Type)
 
@@ -1816,14 +1799,14 @@ treebuilder_handle_data(TreeBuilderObject* self, PyObject* data)
         Py_INCREF(data); self->data = data;
     } else {
         /* more than one item; use a list to collect items */
-        if (PyString_CheckExact(self->data) && Py_REFCNT(self->data) == 1 &&
-            PyString_CheckExact(data) && PyString_GET_SIZE(data) == 1) {
+        if (PyBytes_CheckExact(self->data) && Py_REFCNT(self->data) == 1 &&
+            PyBytes_CheckExact(data) && PyBytes_GET_SIZE(data) == 1) {
             /* expat often generates single character data sections; handle
                the most common case by resizing the existing string... */
-            Py_ssize_t size = PyString_GET_SIZE(self->data);
-            if (_PyString_Resize(&self->data, size + 1) < 0)
+            Py_ssize_t size = PyBytes_GET_SIZE(self->data);
+            if (_PyBytes_Resize(&self->data, size + 1) < 0)
                 return NULL;
-            PyString_AS_STRING(self->data)[size] = PyString_AS_STRING(data)[0];
+            PyBytes_AS_STRING(self->data)[size] = PyBytes_AS_STRING(data)[0];
         } else if (PyList_CheckExact(self->data)) {
             if (PyList_Append(self->data, data) < 0)
                 return NULL;
@@ -2014,19 +1997,35 @@ static PyMethodDef treebuilder_methods[] = {
     {NULL, NULL}
 };
 
-static PyObject*  
-treebuilder_getattr(TreeBuilderObject* self, char* name)
-{
-    return Py_FindMethod(treebuilder_methods, (PyObject*) self, name);
-}
-
-statichere PyTypeObject TreeBuilder_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0, "TreeBuilder", sizeof(TreeBuilderObject), 0,
+static PyTypeObject TreeBuilder_Type = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "TreeBuilder", sizeof(TreeBuilderObject), 0,
     /* methods */
     (destructor)treebuilder_dealloc, /* tp_dealloc */
     0, /* tp_print */
-    (getattrfunc)treebuilder_getattr, /* tp_getattr */
+    0, /* tp_getattr */
+    0, /* tp_setattr */
+    0, /* tp_reserved */
+    0, /* tp_repr */
+    0, /* tp_as_number */
+    0, /* tp_as_sequence */
+    0, /* tp_as_mapping */
+    0, /* tp_hash */
+    0, /* tp_call */
+    0, /* tp_str */
+    0, /* tp_getattro */
+    0, /* tp_setattro */
+    0, /* tp_as_buffer */
+    Py_TPFLAGS_DEFAULT, /* tp_flags */
+    0, /* tp_doc */
+    0, /* tp_traverse */
+    0, /* tp_clear */
+    0, /* tp_richcompare */
+    0, /* tp_weaklistoffset */
+    0, /* tp_iter */
+    0, /* tp_iternext */
+    treebuilder_methods, /* tp_methods */
+    0, /* tp_members */
 };
 
 /* ==================================================================== */
@@ -2067,39 +2066,10 @@ typedef struct {
 
 } XMLParserObject;
 
-staticforward PyTypeObject XMLParser_Type;
+static PyTypeObject XMLParser_Type;
 
 /* helpers */
 
-#if defined(Py_USING_UNICODE)
-LOCAL(int)
-checkstring(const char* string, int size)
-{
-    int i;
-
-    /* check if an 8-bit string contains UTF-8 characters */
-    for (i = 0; i < size; i++)
-        if (string[i] & 0x80)
-            return 1;
-
-    return 0;
-}
-#endif
-
-LOCAL(PyObject*)
-makestring(const char* string, int size)
-{
-    /* convert a UTF-8 string to either a 7-bit ascii string or a
-       Unicode string */
-
-#if defined(Py_USING_UNICODE)
-    if (checkstring(string, size))
-        return PyUnicode_DecodeUTF8(string, size, "strict");
-#endif
-
-    return PyString_FromStringAndSize(string, size);
-}
-
 LOCAL(PyObject*)
 makeuniversal(XMLParserObject* self, const char* string)
 {
@@ -2111,7 +2081,7 @@ makeuniversal(XMLParserObject* self, const char* string)
     PyObject* value;
 
     /* look the 'raw' name up in the names dictionary */
-    key = PyString_FromStringAndSize(string, size);
+    key = PyBytes_FromStringAndSize(string, size);
     if (!key)
         return NULL;
 
@@ -2133,8 +2103,8 @@ makeuniversal(XMLParserObject* self, const char* string)
                 break;
         if (i != size) {
             /* convert to universal name */
-            tag = PyString_FromStringAndSize(NULL, size+1);
-            p = PyString_AS_STRING(tag);
+            tag = PyBytes_FromStringAndSize(NULL, size+1);
+            p = PyBytes_AS_STRING(tag);
             p[0] = '{';
             memcpy(p+1, string, size);
             size++;
@@ -2145,20 +2115,13 @@ makeuniversal(XMLParserObject* self, const char* string)
         }
         
         /* decode universal name */
-#if defined(Py_USING_UNICODE)
-        /* inline makestring, to avoid duplicating the source string if
-           it's not an utf-8 string */
-        p = PyString_AS_STRING(tag);
-        if (checkstring(p, size)) {
-            value = PyUnicode_DecodeUTF8(p, size, "strict");
-            Py_DECREF(tag);
-            if (!value) {
-                Py_DECREF(key);
-                return NULL;
-            }
-        } else
-#endif
-            value = tag; /* use tag as is */
+        p = PyBytes_AS_STRING(tag);
+        value = PyUnicode_DecodeUTF8(p, size, "strict");
+        Py_DECREF(tag);
+        if (!value) {
+            Py_DECREF(key);
+            return NULL;
+        }
 
         /* add to names dictionary */
         if (PyDict_SetItem(self->names, key, value) < 0) {
@@ -2179,7 +2142,7 @@ expat_set_error(const char* message, int line, int column)
     PyObject *position;
     char buffer[256];
 
-    sprintf(buffer, "%s: line %d, column %d", message, line, column);
+    sprintf(buffer, "%.100s: line %d, column %d", message, line, column);
 
     error = PyObject_CallFunction(elementtree_parseerror_obj, "s", buffer);
     if (!error)
@@ -2216,7 +2179,7 @@ expat_default_handler(XMLParserObject* self, const XML_Char* data_in,
     if (data_len < 2 || data_in[0] != '&')
         return;
 
-    key = makestring(data_in + 1, data_len - 2);
+    key = PyUnicode_DecodeUTF8(data_in + 1, data_len - 2, "strict");
     if (!key)
         return;
 
@@ -2234,8 +2197,8 @@ expat_default_handler(XMLParserObject* self, const XML_Char* data_in,
         Py_XDECREF(res);
     } else if (!PyErr_Occurred()) {
         /* Report the first error, not the last */
-        char message[128];
-        sprintf(message, "undefined entity &%.100s;", PyString_AS_STRING(key));
+        char message[128] = "undefined entity ";
+        strncat(message, data_in, data_len < 100?data_len:100);
         expat_set_error(
             message,
             EXPAT(GetErrorLineNumber)(self->parser),
@@ -2267,7 +2230,7 @@ expat_start_handler(XMLParserObject* self, const XML_Char* tag_in,
             return;
         while (attrib_in[0] && attrib_in[1]) {
             PyObject* key = makeuniversal(self, attrib_in[0]);
-            PyObject* value = makestring(attrib_in[1], strlen(attrib_in[1]));
+            PyObject* value = PyUnicode_DecodeUTF8(attrib_in[1], strlen(attrib_in[1]), "strict");
             if (!key || !value) {
                 Py_XDECREF(value);
                 Py_XDECREF(key);
@@ -2316,7 +2279,7 @@ expat_data_handler(XMLParserObject* self, const XML_Char* data_in,
     PyObject* data;
     PyObject* res;
 
-    data = makestring(data_in, data_len);
+    data = PyUnicode_DecodeUTF8(data_in, data_len, "strict");
     if (!data)
         return; /* parser will look for errors */
 
@@ -2363,17 +2326,14 @@ expat_start_ns_handler(XMLParserObject* self, const XML_Char* prefix,
     PyObject* sprefix = NULL;
     PyObject* suri = NULL;
 
-    if (uri)
-        suri = makestring(uri, strlen(uri));
-    else
-        suri = PyString_FromStringAndSize("", 0);
+    suri = PyUnicode_DecodeUTF8(uri, strlen(uri), "strict");
     if (!suri)
         return;
 
     if (prefix)
-        sprefix = makestring(prefix, strlen(prefix));
+        sprefix = PyUnicode_DecodeUTF8(prefix, strlen(prefix), "strict");
     else
-        sprefix = PyString_FromStringAndSize("", 0);
+        sprefix = PyUnicode_FromString("");
     if (!sprefix) {
         Py_DECREF(suri);
         return;
@@ -2402,7 +2362,7 @@ expat_comment_handler(XMLParserObject* self, const XML_Char* comment_in)
     PyObject* res;
 
     if (self->handle_comment) {
-        comment = makestring(comment_in, strlen(comment_in));
+        comment = PyUnicode_DecodeUTF8(comment_in, strlen(comment_in), "strict");
         if (comment) {
             res = PyObject_CallFunction(self->handle_comment, "O", comment);
             Py_XDECREF(res);
@@ -2420,8 +2380,8 @@ expat_pi_handler(XMLParserObject* self, const XML_Char* target_in,
     PyObject* res;
 
     if (self->handle_pi) {
-        target = makestring(target_in, strlen(target_in));
-        data = makestring(data_in, strlen(data_in));
+        target = PyUnicode_DecodeUTF8(target_in, strlen(target_in), "strict");
+        data = PyUnicode_DecodeUTF8(data_in, strlen(data_in), "strict");
         if (target && data) {
             res = PyObject_CallFunction(self->handle_pi, "OO", target, data);
             Py_XDECREF(res);
@@ -2434,7 +2394,6 @@ expat_pi_handler(XMLParserObject* self, const XML_Char* target_in,
     }
 }
 
-#if defined(Py_USING_UNICODE)
 static int
 expat_unknown_encoding_handler(XMLParserObject *self, const XML_Char *name,
                                XML_Encoding *info)
@@ -2455,8 +2414,6 @@ expat_unknown_encoding_handler(XMLParserObject *self, const XML_Char *name,
 
     if (PyUnicode_GET_SIZE(u) != 256) {
         Py_DECREF(u);
-        PyErr_SetString(PyExc_ValueError,
-                        "multi-byte encodings are not supported");
         return XML_STATUS_ERROR;
     }
 
@@ -2473,7 +2430,6 @@ expat_unknown_encoding_handler(XMLParserObject *self, const XML_Char *name,
 
     return XML_STATUS_OK;
 }
-#endif
 
 /* -------------------------------------------------------------------- */
 /* constructor and destructor */
@@ -2580,12 +2536,10 @@ xmlparser(PyObject* self_, PyObject* args, PyObject* kw)
             self->parser,
             (XML_ProcessingInstructionHandler) expat_pi_handler
             );
-#if defined(Py_USING_UNICODE)
     EXPAT(SetUnknownEncodingHandler)(
         self->parser,
         (XML_UnknownEncodingHandler) expat_unknown_encoding_handler, NULL
         );
-#endif
 
     ALLOC(sizeof(XMLParserObject), "create expatparser");
 
@@ -2703,13 +2657,13 @@ xmlparser_parse(XMLParserObject* self, PyObject* args)
             return NULL;
         }
 
-        if (!PyString_CheckExact(buffer) || PyString_GET_SIZE(buffer) == 0) {
+        if (!PyBytes_CheckExact(buffer) || PyBytes_GET_SIZE(buffer) == 0) {
             Py_DECREF(buffer);
             break;
         }
 
         res = expat_parse(
-            self, PyString_AS_STRING(buffer), PyString_GET_SIZE(buffer), 0
+            self, PyBytes_AS_STRING(buffer), PyBytes_GET_SIZE(buffer), 0
             );
 
         Py_DECREF(buffer);
@@ -2764,14 +2718,14 @@ xmlparser_setevents(XMLParserObject* self, PyObject* args)
     target->events = events;
 
     /* clear out existing events */
-    Py_CLEAR(target->start_event_obj);
-    Py_CLEAR(target->end_event_obj);
-    Py_CLEAR(target->start_ns_event_obj);
-    Py_CLEAR(target->end_ns_event_obj);
+    Py_XDECREF(target->start_event_obj); target->start_event_obj = NULL;
+    Py_XDECREF(target->end_event_obj); target->end_event_obj = NULL;
+    Py_XDECREF(target->start_ns_event_obj); target->start_ns_event_obj = NULL;
+    Py_XDECREF(target->end_ns_event_obj); target->end_ns_event_obj = NULL;
 
     if (event_set == Py_None) {
         /* default is "end" only */
-        target->end_event_obj = PyString_FromString("end");
+        target->end_event_obj = PyUnicode_FromString("end");
         Py_RETURN_NONE;
     }
 
@@ -2781,9 +2735,15 @@ xmlparser_setevents(XMLParserObject* self, PyObject* args)
     for (i = 0; i < PyTuple_GET_SIZE(event_set); i++) {
         PyObject* item = PyTuple_GET_ITEM(event_set, i);
         char* event;
-        if (!PyString_Check(item))
+        if (PyUnicode_Check(item)) {
+            event = _PyUnicode_AsString(item);
+            if (event == NULL)
+                goto error;
+        } else if (PyBytes_Check(item))
+            event = PyBytes_AS_STRING(item);
+        else {
             goto error;
-        event = PyString_AS_STRING(item);
+        }
         if (strcmp(event, "start") == 0) {
             Py_INCREF(item);
             target->start_event_obj = item;
@@ -2837,41 +2797,58 @@ static PyMethodDef xmlparser_methods[] = {
 };
 
 static PyObject*  
-xmlparser_getattr(XMLParserObject* self, char* name)
+xmlparser_getattro(XMLParserObject* self, PyObject* nameobj)
 {
-    PyObject* res;
-
-    res = Py_FindMethod(xmlparser_methods, (PyObject*) self, name);
-    if (res)
-        return res;
-
-    PyErr_Clear();
-
-    if (strcmp(name, "entity") == 0)
-        res = self->entity;
-    else if (strcmp(name, "target") == 0)
-        res = self->target;
-    else if (strcmp(name, "version") == 0) {
-        char buffer[100];
-        sprintf(buffer, "Expat %d.%d.%d", XML_MAJOR_VERSION,
+    if (PyUnicode_Check(nameobj)) {
+        PyObject* res;
+        if (PyUnicode_CompareWithASCIIString(nameobj, "entity") == 0)
+            res = self->entity;
+        else if (PyUnicode_CompareWithASCIIString(nameobj, "target") == 0)
+            res = self->target;
+        else if (PyUnicode_CompareWithASCIIString(nameobj, "version") == 0) {
+            return PyUnicode_FromFormat(
+                "Expat %d.%d.%d", XML_MAJOR_VERSION,
                 XML_MINOR_VERSION, XML_MICRO_VERSION);
-        return PyString_FromString(buffer);
-    } else {
-        PyErr_SetString(PyExc_AttributeError, name);
-        return NULL;
-    }
+        }
+        else
+            goto generic;
 
-    Py_INCREF(res);
-    return res;
+        Py_INCREF(res);
+        return res;
+    }
+  generic:
+    return PyObject_GenericGetAttr((PyObject*) self, nameobj);
 }
 
-statichere PyTypeObject XMLParser_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0, "XMLParser", sizeof(XMLParserObject), 0,
+static PyTypeObject XMLParser_Type = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "XMLParser", sizeof(XMLParserObject), 0,
     /* methods */
     (destructor)xmlparser_dealloc, /* tp_dealloc */
     0, /* tp_print */
-    (getattrfunc)xmlparser_getattr, /* tp_getattr */
+    0, /* tp_getattr */
+    0, /* tp_setattr */
+    0, /* tp_reserved */
+    0, /* tp_repr */
+    0, /* tp_as_number */
+    0, /* tp_as_sequence */
+    0, /* tp_as_mapping */
+    0, /* tp_hash */
+    0, /* tp_call */
+    0, /* tp_str */
+    (getattrofunc)xmlparser_getattro, /* tp_getattro */
+    0, /* tp_setattro */
+    0, /* tp_as_buffer */
+    Py_TPFLAGS_DEFAULT, /* tp_flags */
+    0, /* tp_doc */
+    0, /* tp_traverse */
+    0, /* tp_clear */
+    0, /* tp_richcompare */
+    0, /* tp_weaklistoffset */
+    0, /* tp_iter */
+    0, /* tp_iternext */
+    xmlparser_methods, /* tp_methods */
+    0, /* tp_members */
 };
 
 #endif
@@ -2890,28 +2867,51 @@ static PyMethodDef _functions[] = {
     {NULL, NULL}
 };
 
-DL_EXPORT(void)
-init_elementtree(void)
+
+static struct PyModuleDef _elementtreemodule = {
+        PyModuleDef_HEAD_INIT,
+        "_elementtree",
+        NULL,
+        -1,
+        _functions,
+        NULL,
+        NULL,
+        NULL,
+        NULL
+};
+
+PyMODINIT_FUNC
+PyInit__elementtree(void)
 {
     PyObject* m;
     PyObject* g;
     char* bootstrap;
 
-    /* Patch object type */
-    Py_TYPE(&Element_Type) = Py_TYPE(&TreeBuilder_Type) = &PyType_Type;
+    /* Initialize object types */
+    if (PyType_Ready(&TreeBuilder_Type) < 0)
+        return NULL;
+    if (PyType_Ready(&Element_Type) < 0)
+        return NULL;
 #if defined(USE_EXPAT)
-    Py_TYPE(&XMLParser_Type) = &PyType_Type;
+    if (PyType_Ready(&XMLParser_Type) < 0)
+        return NULL;
 #endif
 
-    m = Py_InitModule("_elementtree", _functions);
+    m = PyModule_Create(&_elementtreemodule);
     if (!m)
-        return;
+        return NULL;
+
+    /* The code below requires that the module gets already added
+       to sys.modules. */
+    PyDict_SetItemString(PyImport_GetModuleDict(),
+                         _elementtreemodule.m_name,
+                         m);
 
     /* python glue code */
 
     g = PyDict_New();
     if (!g)
-        return;
+        return NULL;
 
     PyDict_SetItemString(g, "__builtins__", PyEval_GetBuiltins());
 
@@ -2939,8 +2939,8 @@ init_elementtree(void)
         "  element = cElementTree.Element(ET.Comment)\n"
         "  element.text = text\n"
         "  return element\n"
-        " def __cmp__(self, other):\n"
-        "  return cmp(ET.Comment, other)\n"
+        " def __eq__(self, other):\n"
+        "  return ET.Comment == other\n"
         "cElementTree.Comment = CommentProxy()\n"
 
         "class ElementTree(ET.ElementTree):\n" /* public */
@@ -2948,7 +2948,7 @@ init_elementtree(void)
         "    close_source = False\n"
         "    if not hasattr(source, 'read'):\n"
         "      source = open(source, 'rb')\n"
-        "      close_source = False\n"
+        "      close_source = True\n"
         "    try:\n"
         "      if parser is not None:\n"
         "        while 1:\n"
@@ -2990,7 +2990,7 @@ init_elementtree(void)
         "  return tree\n"
         "cElementTree.parse = parse\n"
 
-        "class iterparse(object):\n"
+        "class iterparse:\n"
         " root = None\n"
         " def __init__(self, file, events=None):\n"
         "  self._close_file = False\n"
@@ -3005,7 +3005,7 @@ init_elementtree(void)
         "  b = cElementTree.TreeBuilder()\n"
         "  self._parser = cElementTree.XMLParser(b)\n"
         "  self._parser._setevents(self._events, events)\n"
-        " def next(self):\n"
+        " def __next__(self):\n"
         "  while 1:\n"
         "    try:\n"
         "      item = self._events[self._index]\n"
@@ -3045,8 +3045,8 @@ init_elementtree(void)
         "  if text:\n"
         "    element.text = element.text + ' ' + text\n"
         "  return element\n"
-        " def __cmp__(self, other):\n"
-        "  return cmp(ET.PI, other)\n"
+        " def __eq__(self, other):\n"
+        "  return ET.PI == other\n"
         "cElementTree.PI = cElementTree.ProcessingInstruction = PIProxy()\n"
 
         "def XML(text):\n" /* public */
@@ -3085,7 +3085,7 @@ init_elementtree(void)
        );
 
     if (!PyRun_String(bootstrap, Py_file_input, g, NULL))
-        return;
+        return NULL;
 
     elementpath_obj = PyDict_GetItemString(g, "ElementPath");
 
@@ -3114,8 +3114,13 @@ init_elementtree(void)
             expat_capi->size < sizeof(struct PyExpat_CAPI) ||
             expat_capi->MAJOR_VERSION != XML_MAJOR_VERSION ||
             expat_capi->MINOR_VERSION != XML_MINOR_VERSION ||
-            expat_capi->MICRO_VERSION != XML_MICRO_VERSION)
-            expat_capi = NULL;
+            expat_capi->MICRO_VERSION != XML_MICRO_VERSION) {
+            PyErr_SetString(PyExc_ImportError,
+                            "pyexpat version is incompatible");
+            return NULL;
+        }
+    } else {
+        return NULL;
     }
 #endif
 
@@ -3124,4 +3129,6 @@ init_elementtree(void)
         );
     Py_INCREF(elementtree_parseerror_obj);
     PyModule_AddObject(m, "ParseError", elementtree_parseerror_obj);
+
+    return m;
 }
