@@ -2,8 +2,6 @@
    Under Unix, the file descriptors are small integers.
    Under Win32, select only exists for sockets, and sockets may
    have any value except INVALID_SOCKET.
-   Under BeOS, we suffer the same dichotomy as Win32; sockets can be anything
-   >= 0.
 */
 
 #include "Python.h"
@@ -47,12 +45,11 @@ extern void bzero(void *, int);
 #endif
 
 #ifdef MS_WINDOWS
-#  include <winsock2.h>
+#  define WIN32_LEAN_AND_MEAN
+#  include <winsock.h>
 #else
 #  define SOCKET int
-#  ifdef __BEOS__
-#    include <net/socket.h>
-#  elif defined(__VMS)
+#  if defined(__VMS)
 #    include <socket.h>
 #  endif
 #endif
@@ -71,7 +68,8 @@ reap_obj(pylist fd2obj[FD_SETSIZE + 1])
 {
     int i;
     for (i = 0; i < FD_SETSIZE + 1 && fd2obj[i].sentinel >= 0; i++) {
-        Py_CLEAR(fd2obj[i].obj);
+        Py_XDECREF(fd2obj[i].obj);
+        fd2obj[i].obj = NULL;
     }
     fd2obj[0].sentinel = -1;
 }
@@ -83,9 +81,9 @@ reap_obj(pylist fd2obj[FD_SETSIZE + 1])
 static int
 seq2set(PyObject *seq, fd_set *set, pylist fd2obj[FD_SETSIZE + 1])
 {
-    int i;
     int max = -1;
     int index = 0;
+    Py_ssize_t i;
     PyObject* fast_seq = NULL;
     PyObject* o = NULL;
 
@@ -101,7 +99,7 @@ seq2set(PyObject *seq, fd_set *set, pylist fd2obj[FD_SETSIZE + 1])
 
         /* any intervening fileno() calls could decr this refcnt */
         if (!(o = PySequence_Fast_GET_ITEM(fast_seq, i)))
-            return -1;
+            goto finally;
 
         Py_INCREF(o);
         v = PyObject_AsFileDescriptor( o );
@@ -317,7 +315,6 @@ typedef struct {
     int ufd_uptodate;
     int ufd_len;
     struct pollfd *ufds;
-    int poll_running;
 } pollObject;
 
 static PyTypeObject poll_Type;
@@ -345,8 +342,8 @@ update_ufd_array(pollObject *self)
     while (PyDict_Next(self->dict, &pos, &key, &value)) {
         assert(i < self->ufd_len);
         /* Never overflow */
-        self->ufds[i].fd = (int)PyInt_AsLong(key);
-        self->ufds[i].events = (short)(unsigned short)PyInt_AsLong(value);
+        self->ufds[i].fd = (int)PyLong_AsLong(key);
+        self->ufds[i].events = (short)PyLong_AsLong(value);
         i++;
     }
     assert(i == self->ufd_len);
@@ -354,24 +351,6 @@ update_ufd_array(pollObject *self)
     return 1;
 }
 
-static int
-ushort_converter(PyObject *obj, void *ptr)
-{
-    unsigned long uval;
-
-    uval = PyLong_AsUnsignedLong(obj);
-    if (uval == (unsigned long)-1 && PyErr_Occurred())
-        return 0;
-    if (uval > USHRT_MAX) {
-        PyErr_SetString(PyExc_OverflowError,
-                        "Python int too large for C unsigned short");
-        return 0;
-    }
-
-    *(unsigned short *)ptr = Py_SAFE_DOWNCAST(uval, unsigned long, unsigned short);
-    return 1;
-}
-
 PyDoc_STRVAR(poll_register_doc,
 "register(fd [, eventmask] ) -> None\n\n\
 Register a file descriptor with the polling object.\n\
@@ -384,21 +363,22 @@ poll_register(pollObject *self, PyObject *args)
 {
     PyObject *o, *key, *value;
     int fd;
-    unsigned short events = POLLIN | POLLPRI | POLLOUT;
+    short events = POLLIN | POLLPRI | POLLOUT;
     int err;
 
-    if (!PyArg_ParseTuple(args, "O|O&:register", &o, ushort_converter, &events))
+    if (!PyArg_ParseTuple(args, "O|h:register", &o, &events)) {
         return NULL;
+    }
 
     fd = PyObject_AsFileDescriptor(o);
     if (fd == -1) return NULL;
 
     /* Add entry to the internal dictionary: the key is the
        file descriptor, and the value is the event mask. */
-    key = PyInt_FromLong(fd);
+    key = PyLong_FromLong(fd);
     if (key == NULL)
         return NULL;
-    value = PyInt_FromLong(events);
+    value = PyLong_FromLong(events);
     if (value == NULL) {
         Py_DECREF(key);
         return NULL;
@@ -426,26 +406,27 @@ static PyObject *
 poll_modify(pollObject *self, PyObject *args)
 {
     PyObject *o, *key, *value;
-    int fd;
-    unsigned short events;
+    int fd, events;
     int err;
 
-    if (!PyArg_ParseTuple(args, "OO&:modify", &o, ushort_converter, &events))
+    if (!PyArg_ParseTuple(args, "Oi:modify", &o, &events)) {
         return NULL;
+    }
 
     fd = PyObject_AsFileDescriptor(o);
     if (fd == -1) return NULL;
 
     /* Modify registered fd */
-    key = PyInt_FromLong(fd);
+    key = PyLong_FromLong(fd);
     if (key == NULL)
         return NULL;
     if (PyDict_GetItem(self->dict, key) == NULL) {
         errno = ENOENT;
         PyErr_SetFromErrno(PyExc_IOError);
+        Py_DECREF(key);
         return NULL;
     }
-    value = PyInt_FromLong(events);
+    value = PyLong_FromLong(events);
     if (value == NULL) {
         Py_DECREF(key);
         return NULL;
@@ -478,7 +459,7 @@ poll_unregister(pollObject *self, PyObject *o)
         return NULL;
 
     /* Check whether the fd is already in the array */
-    key = PyInt_FromLong(fd);
+    key = PyLong_FromLong(fd);
     if (key == NULL)
         return NULL;
 
@@ -521,36 +502,25 @@ poll_poll(pollObject *self, PyObject *args)
         return NULL;
     }
     else {
-        tout = PyNumber_Int(tout);
+        tout = PyNumber_Long(tout);
         if (!tout)
             return NULL;
-        timeout = _PyInt_AsInt(tout);
+        timeout = _PyLong_AsInt(tout);
         Py_DECREF(tout);
         if (timeout == -1 && PyErr_Occurred())
             return NULL;
     }
 
-    /* Avoid concurrent poll() invocation, issue 8865 */
-    if (self->poll_running) {
-        PyErr_SetString(PyExc_RuntimeError,
-                        "concurrent poll() invocation");
-        return NULL;
-    }
-
     /* Ensure the ufd array is up to date */
     if (!self->ufd_uptodate)
         if (update_ufd_array(self) == 0)
             return NULL;
 
-    self->poll_running = 1;
-
     /* call poll() */
     Py_BEGIN_ALLOW_THREADS
     poll_result = poll(self->ufds, self->ufd_len, timeout);
     Py_END_ALLOW_THREADS
 
-    self->poll_running = 0;
-
     if (poll_result < 0) {
         PyErr_SetFromErrno(SelectError);
         return NULL;
@@ -573,7 +543,7 @@ poll_poll(pollObject *self, PyObject *args)
             value = PyTuple_New(2);
             if (value == NULL)
                 goto error;
-            num = PyInt_FromLong(self->ufds[i].fd);
+            num = PyLong_FromLong(self->ufds[i].fd);
             if (num == NULL) {
                 Py_DECREF(value);
                 goto error;
@@ -584,7 +554,7 @@ poll_poll(pollObject *self, PyObject *args)
                is a 16-bit short, and IBM assigned POLLNVAL
                to be 0x8000, so the conversion to int results
                in a negative number. See SF bug #923315. */
-            num = PyInt_FromLong(self->ufds[i].revents & 0xffff);
+            num = PyLong_FromLong(self->ufds[i].revents & 0xffff);
             if (num == NULL) {
                 Py_DECREF(value);
                 goto error;
@@ -627,7 +597,6 @@ newPollObject(void)
        array pointed to by ufds matches the contents of the dictionary. */
     self->ufd_uptodate = 0;
     self->ufds = NULL;
-    self->poll_running = 0;
     self->dict = PyDict_New();
     if (self->dict == NULL) {
         Py_DECREF(self);
@@ -645,12 +614,6 @@ poll_dealloc(pollObject *self)
     PyObject_Del(self);
 }
 
-static PyObject *
-poll_getattr(pollObject *self, char *name)
-{
-    return Py_FindMethod(poll_methods, (PyObject *)self, name);
-}
-
 static PyTypeObject poll_Type = {
     /* The ob_type field must be initialized in the module init function
      * to be portable to Windows without using C++. */
@@ -661,14 +624,28 @@ static PyTypeObject poll_Type = {
     /* methods */
     (destructor)poll_dealloc, /*tp_dealloc*/
     0,                          /*tp_print*/
-    (getattrfunc)poll_getattr, /*tp_getattr*/
+    0,                          /*tp_getattr*/
     0,                      /*tp_setattr*/
-    0,                          /*tp_compare*/
+    0,                          /*tp_reserved*/
     0,                          /*tp_repr*/
     0,                          /*tp_as_number*/
     0,                          /*tp_as_sequence*/
     0,                          /*tp_as_mapping*/
     0,                          /*tp_hash*/
+    0,                          /*tp_call*/
+    0,                          /*tp_str*/
+    0,                          /*tp_getattro*/
+    0,                          /*tp_setattro*/
+    0,                          /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT,         /*tp_flags*/
+    0,                          /*tp_doc*/
+    0,                          /*tp_traverse*/
+    0,                          /*tp_clear*/
+    0,                          /*tp_richcompare*/
+    0,                          /*tp_weaklistoffset*/
+    0,                          /*tp_iter*/
+    0,                          /*tp_iternext*/
+    poll_methods,               /*tp_methods*/
 };
 
 PyDoc_STRVAR(poll_doc,
@@ -844,7 +821,7 @@ pyepoll_fileno(pyEpoll_Object *self)
 {
     if (self->epfd < 0)
         return pyepoll_err_closed();
-    return PyInt_FromLong(self->epfd);
+    return PyLong_FromLong(self->epfd);
 }
 
 PyDoc_STRVAR(pyepoll_fileno_doc,
@@ -1106,7 +1083,7 @@ static PyTypeObject pyEpoll_Type = {
     0,                                                  /* tp_print */
     0,                                                  /* tp_getattr */
     0,                                                  /* tp_setattr */
-    0,                                                  /* tp_compare */
+    0,                                                  /* tp_reserved */
     0,                                                  /* tp_repr */
     0,                                                  /* tp_as_number */
     0,                                                  /* tp_as_sequence */
@@ -1232,23 +1209,6 @@ static PyTypeObject kqueue_queue_Type;
 #   error uintptr_t does not match int, long, or long long!
 #endif
 
-/*
- * kevent is not standard and its members vary across BSDs.
- */
-#if !defined(__OpenBSD__)
-#   define IDENT_TYPE	T_UINTPTRT
-#   define IDENT_CAST	Py_intptr_t
-#   define DATA_TYPE	T_INTPTRT
-#   define DATA_FMT_UNIT INTPTRT_FMT_UNIT
-#   define IDENT_AsType	PyLong_AsUintptr_t
-#else
-#   define IDENT_TYPE	T_UINT
-#   define IDENT_CAST	int
-#   define DATA_TYPE	T_INT
-#   define DATA_FMT_UNIT "i"
-#   define IDENT_AsType	PyLong_AsUnsignedLong
-#endif
-
 /* Unfortunately, we can't store python objects in udata, because
  * kevents in the kernel can be removed without warning, which would
  * forever lose the refcount on the object stored with it.
@@ -1256,11 +1216,11 @@ static PyTypeObject kqueue_queue_Type;
 
 #define KQ_OFF(x) offsetof(kqueue_event_Object, x)
 static struct PyMemberDef kqueue_event_members[] = {
-    {"ident",           IDENT_TYPE,     KQ_OFF(e.ident)},
+    {"ident",           T_UINTPTRT,     KQ_OFF(e.ident)},
     {"filter",          T_SHORT,        KQ_OFF(e.filter)},
     {"flags",           T_USHORT,       KQ_OFF(e.flags)},
     {"fflags",          T_UINT,         KQ_OFF(e.fflags)},
-    {"data",            DATA_TYPE,      KQ_OFF(e.data)},
+    {"data",            T_INTPTRT,      KQ_OFF(e.data)},
     {"udata",           T_UINTPTRT,     KQ_OFF(e.udata)},
     {NULL} /* Sentinel */
 };
@@ -1277,7 +1237,7 @@ kqueue_event_repr(kqueue_event_Object *s)
         "data=0x%zd udata=%p>",
         (size_t)(s->e.ident), s->e.filter, s->e.flags,
         s->e.fflags, (Py_ssize_t)(s->e.data), s->e.udata);
-    return PyString_FromString(buf);
+    return PyUnicode_FromString(buf);
 }
 
 static int
@@ -1286,7 +1246,7 @@ kqueue_event_init(kqueue_event_Object *self, PyObject *args, PyObject *kwds)
     PyObject *pfd;
     static char *kwlist[] = {"ident", "filter", "flags", "fflags",
                              "data", "udata", NULL};
-    static char *fmt = "O|hHI" DATA_FMT_UNIT UINTPTRT_FMT_UNIT ":kevent";
+    static char *fmt = "O|hhi" INTPTRT_FMT_UNIT UINTPTRT_FMT_UNIT ":kevent";
 
     EV_SET(&(self->e), 0, EVFILT_READ, EV_ADD, 0, 0, 0); /* defaults */
 
@@ -1296,12 +1256,8 @@ kqueue_event_init(kqueue_event_Object *self, PyObject *args, PyObject *kwds)
         return -1;
     }
 
-    if (PyLong_Check(pfd)
-#if IDENT_TYPE == T_UINT
-	&& PyLong_AsUnsignedLong(pfd) <= UINT_MAX
-#endif
-    ) {
-        self->e.ident = IDENT_AsType(pfd);
+    if (PyLong_Check(pfd)) {
+        self->e.ident = PyLong_AsUintptr_t(pfd);
     }
     else {
         self->e.ident = PyObject_AsFileDescriptor(pfd);
@@ -1329,10 +1285,10 @@ kqueue_event_richcompare(kqueue_event_Object *s, kqueue_event_Object *o,
             Py_TYPE(s)->tp_name, Py_TYPE(o)->tp_name);
         return NULL;
     }
-    if (((result = (IDENT_CAST)(s->e.ident - o->e.ident)) == 0) &&
+    if (((result = s->e.ident - o->e.ident) == 0) &&
         ((result = s->e.filter - o->e.filter) == 0) &&
         ((result = s->e.flags - o->e.flags) == 0) &&
-        ((result = (int)(s->e.fflags - o->e.fflags)) == 0) &&
+        ((result = s->e.fflags - o->e.fflags) == 0) &&
         ((result = s->e.data - o->e.data) == 0) &&
         ((result = s->e.udata - o->e.udata) == 0)
        ) {
@@ -1371,7 +1327,7 @@ static PyTypeObject kqueue_event_Type = {
     0,                                                  /* tp_print */
     0,                                                  /* tp_getattr */
     0,                                                  /* tp_setattr */
-    0,                                                  /* tp_compare */
+    0,                                                  /* tp_reserved */
     (reprfunc)kqueue_event_repr,                        /* tp_repr */
     0,                                                  /* tp_as_number */
     0,                                                  /* tp_as_sequence */
@@ -1504,7 +1460,7 @@ kqueue_queue_fileno(kqueue_queue_Object *self)
 {
     if (self->kqfd < 0)
         return kqueue_queue_err_closed();
-    return PyInt_FromLong(self->kqfd);
+    return PyLong_FromLong(self->kqfd);
 }
 
 PyDoc_STRVAR(kqueue_queue_fileno_doc,
@@ -1725,7 +1681,7 @@ static PyTypeObject kqueue_queue_Type = {
     0,                                                  /* tp_print */
     0,                                                  /* tp_getattr */
     0,                                                  /* tp_setattr */
-    0,                                                  /* tp_compare */
+    0,                                                  /* tp_reserved */
     0,                                                  /* tp_repr */
     0,                                                  /* tp_as_number */
     0,                                                  /* tp_as_sequence */
@@ -1799,13 +1755,26 @@ PyDoc_STRVAR(module_doc,
 *** IMPORTANT NOTICE ***\n\
 On Windows and OpenVMS, only sockets are supported; on Unix, all file descriptors.");
 
+
+static struct PyModuleDef selectmodule = {
+    PyModuleDef_HEAD_INIT,
+    "select",
+    module_doc,
+    -1,
+    select_methods,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+};
+
 PyMODINIT_FUNC
-initselect(void)
+PyInit_select(void)
 {
     PyObject *m;
-    m = Py_InitModule3("select", select_methods, module_doc);
+    m = PyModule_Create(&selectmodule);
     if (m == NULL)
-        return;
+        return NULL;
 
     SelectError = PyErr_NewException("select.error", NULL, NULL);
     Py_INCREF(SelectError);
@@ -1829,7 +1798,8 @@ initselect(void)
 #else
     {
 #endif
-        Py_TYPE(&poll_Type) = &PyType_Type;
+        if (PyType_Ready(&poll_Type) < 0)
+            return NULL;
         PyModule_AddIntConstant(m, "POLLIN", POLLIN);
         PyModule_AddIntConstant(m, "POLLPRI", POLLPRI);
         PyModule_AddIntConstant(m, "POLLOUT", POLLOUT);
@@ -1858,7 +1828,7 @@ initselect(void)
 #ifdef HAVE_EPOLL
     Py_TYPE(&pyEpoll_Type) = &PyType_Type;
     if (PyType_Ready(&pyEpoll_Type) < 0)
-        return;
+        return NULL;
 
     Py_INCREF(&pyEpoll_Type);
     PyModule_AddObject(m, "epoll", (PyObject *) &pyEpoll_Type);
@@ -1885,14 +1855,14 @@ initselect(void)
     kqueue_event_Type.tp_new = PyType_GenericNew;
     Py_TYPE(&kqueue_event_Type) = &PyType_Type;
     if(PyType_Ready(&kqueue_event_Type) < 0)
-        return;
+        return NULL;
 
     Py_INCREF(&kqueue_event_Type);
     PyModule_AddObject(m, "kevent", (PyObject *)&kqueue_event_Type);
 
     Py_TYPE(&kqueue_queue_Type) = &PyType_Type;
     if(PyType_Ready(&kqueue_queue_Type) < 0)
-        return;
+        return NULL;
     Py_INCREF(&kqueue_queue_Type);
     PyModule_AddObject(m, "kqueue", (PyObject *)&kqueue_queue_Type);
 
@@ -1953,4 +1923,5 @@ initselect(void)
 #endif
 
 #endif /* HAVE_KQUEUE */
+    return m;
 }
